@@ -1406,7 +1406,7 @@ static void __init axp_db_init(void)
 	cpu_fabric_common_init();
 
 	/* Select appropriate Board ID for Machine */
-#ifdef CONFIG_ARMADA_XP_REV_A0
+#if defined(CONFIG_ARMADA_XP_REV_A0) || defined(CONFIG_ARMADA_XP_REV_B0)
 	gBoardId = DB_88F78XX0_BP_REV2_ID;
 #else
 	gBoardId = DB_88F78XX0_BP_ID;
@@ -1811,16 +1811,14 @@ static void __init axp_rdsrv_init(void)
 	return;
 }
 
-/*MACHINE_START(ARMADA_XP_RDSRV, "Marvell Armada XP Server Board")
-		/ MAINTAINER("MARVELL") 
-		.phys_io	= INTER_REGS_PHYS_BASE,
-  .io_pg_offst	= ((INTER_REGS_BASE) >> 18) & 0xfffc,
-					.boot_params	= 0x00000100,
+MACHINE_START(ARMADA_XP_RDSRV, "Marvell Armada XP Server Board")
+		/* MAINTAINER("MARVELL") */
+	 .atag_offset	= (PHYS_OFFSET + 0x100),
 	 .map_io		= axp_map_io,
-  .init_irq	= axp_init_irq,
-  .timer		= &axp_timer,
-  .init_machine	= axp_rdsrv_init,
-  MACHINE_END */
+	 .init_irq	= axp_init_irq,
+	 .timer		= &axp_timer,
+	 .init_machine	= axp_rdsrv_init,
+MACHINE_END
 
 /*****************************************************************************
  * FPGA BOARD: Main Initialization
