@@ -13,8 +13,6 @@
 
 //#define pcibios_assign_all_busses()	1
 
-#define PCIBIOS_MIN_IO			0x1000
-#define PCIBIOS_MIN_MEM			0x01000000
 #define PCIMEM_BASE			PEX0_MEM_PHYS_BASE /* mem base for VGA */
 
 
