@@ -14,7 +14,13 @@
 
 #include "boardEnv/mvBoardEnvLib.h"
 
-#define LSP_VERSION    "AXP_3.0.0-rc1"
+#ifdef CONFIG_MV_SUPPORT_64KB_PAGE_SIZE
+#define LSP_PG_SZ_VER  " (Large Page)"
+#else
+#define LSP_PG_SZ_VER  ""
+#endif
+
+#define LSP_VERSION    "AXP_2.0.6" LSP_PG_SZ_VER
 
 static inline void arch_idle(void)
 {
